// (C) 2001-2017 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel MegaCore Function License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


// *********************************************************************
//
// DisplayPort IP Core
// 
// Description
// 
// Top level module for DisplayPort IP
// This module instantiates the Bitec DP core and the Custom PHY IP
// It should be delivered as clear text to allow users who need to customize 
// their XCVR implementation (for example to pack unrelated RX/TX simplex
// protocols)
//
// *********************************************************************

// synthesis translate_off
`timescale 1ps / 1ps
// synthesis translate_on
`default_nettype none

// top-level module name is autogenerated by Qsys to enable different
// parameterizations of the PHY IP 
module subsys_dp_bitec_dp_0_altera_dp_170_mvrvugy #(
    parameter device_family = "Stratix V",

    parameter TX_SUPPORT_DP = 1,
    parameter TX_MAX_LINK_RATE = 10,
    parameter TX_MAX_LANE_COUNT = 4,
    parameter TX_SUPPORT_ANALOG_RECONFIG = 0,
    parameter TX_SCRAMBLER_SEED = 16'hffff,
    parameter TX_SUPPORT_SS = 1,
    parameter TX_PIXELS_PER_CLOCK = 1,      // replaces RX_ODD_EVEN_PIXEL
    parameter TX_SYMBOLS_PER_CLOCK = 2,
    parameter TX_VIDEO_IM_ENABLE = 0,		// Enable Video input Image port instead of traditional VS/HS/DE interface
    parameter TX_SUPPORT_18BPP = 1,
    parameter TX_SUPPORT_24BPP = 1,
    parameter TX_SUPPORT_30BPP = 1,
    parameter TX_SUPPORT_36BPP = 1,
    parameter TX_SUPPORT_48BPP = 1,
    parameter TX_SUPPORT_YCBCR422_16BPP = 1,
    parameter TX_SUPPORT_YCBCR422_20BPP = 1,
    parameter TX_SUPPORT_YCBCR422_24BPP = 1,
    parameter TX_SUPPORT_YCBCR422_32BPP = 1,
    parameter TX_SUPPORT_YCBCR420_12BPP = 1,
    parameter TX_SUPPORT_YCBCR420_15BPP = 1,
    parameter TX_SUPPORT_YCBCR420_18BPP = 1,
    parameter TX_SUPPORT_YCBCR420_24BPP = 1,
    parameter TX_VIDEO_BPS = 16,            // Bitec renamed to TX_VIDEO_BPC
    parameter TX_SUPPORT_AUTOMATED_TEST = 0,
    parameter TX_SUPPORT_GTC = 0,
    parameter TX_AUX_DEBUG = 0,
    parameter TX_SUPPORT_AUDIO = 1,
    parameter TX_AUDIO_CHANS = 2,
    parameter TX_SUPPORT_MST = 0,
    parameter TX_MAX_NUM_OF_STREAMS = 1,

    parameter RX_SUPPORT_DP = 1,
    parameter RX_MAX_LINK_RATE = 10,
    parameter RX_MAX_LANE_COUNT = 4,
    parameter RX_SCRAMBLER_SEED = 16'hffff,
    parameter RX_SUPPORT_EDP = 0,
    parameter RX_SUPPORT_SS = 1,
    parameter RX_PIXELS_PER_CLOCK = 1,      // replaces RX_ODD_EVEN_PIXEL
    parameter RX_SYMBOLS_PER_CLOCK = 2,
    parameter RX_SUPPORT_AUTOMATED_TEST = 0,
    parameter RX_SUPPORT_GTC = 0,
    parameter RX_SUPPORT_18BPP = 1,
    parameter RX_SUPPORT_24BPP = 1,
    parameter RX_SUPPORT_30BPP = 1,
    parameter RX_SUPPORT_36BPP = 1,
    parameter RX_SUPPORT_48BPP = 1,
    parameter RX_SUPPORT_YCBCR422_16BPP = 1,
    parameter RX_SUPPORT_YCBCR422_20BPP = 1,
    parameter RX_SUPPORT_YCBCR422_24BPP = 1,
    parameter RX_SUPPORT_YCBCR422_32BPP = 1,
    parameter RX_SUPPORT_YCBCR420_12BPP = 1,
    parameter RX_SUPPORT_YCBCR420_15BPP = 1,
    parameter RX_SUPPORT_YCBCR420_18BPP = 1,
    parameter RX_SUPPORT_YCBCR420_24BPP = 1,
    parameter RX_VIDEO_BPS = 16,            // Bitec renamed to RX_VIDEO_BPC
    parameter RX_AUX_DEBUG = 0,
    parameter RX_IEEE_OUI = 1,
    parameter RX_AUX_GPU = 0,
    parameter RX_EXPORT_MSA = 0,
    parameter RX_SUPPORT_AUDIO = 1,
    parameter RX_AUDIO_CHANS = 2,
    parameter RX_SUPPORT_MST = 0,
    parameter RX_MAX_NUM_OF_STREAMS = 1
)(
    //************************************************************************
    // Main clock domain
    //
    // Clock is clk, reset is reset
    //************************************************************************
    input  wire reset, 
    input  wire clk, 

    // Management interfaces

    // TX Avalon-MM slave (tx_mgmt)
    input  wire [8:0]  tx_mgmt_address,
    input  wire        tx_mgmt_chipselect,
    input  wire        tx_mgmt_read,
    input  wire        tx_mgmt_write,
    input  wire [31:0] tx_mgmt_writedata,
    output wire [31:0] tx_mgmt_readdata,
    output wire        tx_mgmt_waitrequest, 
    output wire        tx_mgmt_irq,

    // RX Avalon-MM slave (rx_mgmt)
    input  wire [8:0]  rx_mgmt_address,
    input  wire        rx_mgmt_chipselect,
    input  wire        rx_mgmt_read,
    input  wire        rx_mgmt_write,
    input  wire [31:0] rx_mgmt_writedata,
    output wire [31:0] rx_mgmt_readdata,
    output wire        rx_mgmt_waitrequest,
    output wire        rx_mgmt_irq,

    //************************************************************************
    // XCVR Mgmt clock domain
    //
    // Clock is xcvr_mgmt_clk, reset is reset
    //************************************************************************
    input  wire       xcvr_mgmt_clk, // 100 MHz
    input  wire       clk_cal, // 50 MHz

    // TX XCVR reconfig interface (tx_reconfig)
    output wire [1:0] tx_link_rate,
    output wire [7:0] tx_link_rate_8bits,
    output wire       tx_reconfig_req, 
    input  wire       tx_reconfig_ack, 
    input  wire       tx_reconfig_busy, 

    // TX Analog reconfig interface (tx_analog_reconfig)
    output wire [(TX_MAX_LANE_COUNT*2)-1:0] tx_vod, 
    output wire [(TX_MAX_LANE_COUNT*2)-1:0] tx_emp, 
    output wire                             tx_analog_reconfig_req, 
    input  wire                             tx_analog_reconfig_ack, 
    input  wire                             tx_analog_reconfig_busy, 

    // RX XCVR reconfig interface (rx_reconfig)
    output wire [1:0] rx_link_rate,
    output wire [7:0] rx_link_rate_8bits,
    output wire       rx_reconfig_req, 
    input  wire       rx_reconfig_ack, 
    input  wire       rx_reconfig_busy, 
        
    // XCVR interface
    output wire [(TX_MAX_LANE_COUNT*10*TX_SYMBOLS_PER_CLOCK)-1:0] tx_parallel_data,
    output wire                                                   tx_pll_powerdown,
    output wire [TX_MAX_LANE_COUNT-1:0]                           tx_analogreset,
    output wire [TX_MAX_LANE_COUNT-1:0]                           tx_digitalreset,
    input wire  [TX_MAX_LANE_COUNT-1:0]                           tx_cal_busy,
    input wire  [TX_MAX_LANE_COUNT-1:0]                           tx_std_clkout,
    input wire                                                    tx_pll_locked,
    input wire  [(RX_MAX_LANE_COUNT*10*RX_SYMBOLS_PER_CLOCK)-1:0] rx_parallel_data,
    input wire  [RX_MAX_LANE_COUNT-1:0]                           rx_std_clkout,
    input wire  [RX_MAX_LANE_COUNT-1:0]                           rx_is_lockedtoref,
    input wire  [RX_MAX_LANE_COUNT-1:0]                           rx_is_lockedtodata,
    output wire							  rx_restart,
    output wire [RX_MAX_LANE_COUNT-1:0]                           rx_bitslip,
    input wire  [RX_MAX_LANE_COUNT-1:0]                           rx_cal_busy,
    output wire [RX_MAX_LANE_COUNT-1:0]                           rx_analogreset,
    output wire [RX_MAX_LANE_COUNT-1:0]                           rx_digitalreset,
    output wire [RX_MAX_LANE_COUNT-1:0]                           rx_set_locktoref,
    output wire [RX_MAX_LANE_COUNT-1:0]                           rx_set_locktodata,

    //************************************************************************
    // RX FIFO overflow signals 
    //
    //************************************************************************

    output wire rx_vid_overflow,
    output wire rx1_vid_overflow,
    output wire rx2_vid_overflow,
    output wire rx3_vid_overflow,

    //************************************************************************
    // Video clock domain
    //
    // TX Clock is tx_vid_clk, reset is reset
    // RX Clock is rx_vid_clk, reset is reset
    //************************************************************************
    input wire tx_vid_clk,  // DP Source Pixel clock frequency
    input wire tx1_vid_clk,  // DP Source Pixel clock frequency
    input wire tx2_vid_clk,  // DP Source Pixel clock frequency
    input wire tx3_vid_clk,  // DP Source Pixel clock frequency

    // TX Video input (tx_video_in)
    input wire [(TX_VIDEO_BPS*3*TX_PIXELS_PER_CLOCK)-1:0] tx_vid_data,
    input wire [TX_PIXELS_PER_CLOCK-1:0]                  tx_vid_v_sync,
    input wire [TX_PIXELS_PER_CLOCK-1:0]                  tx_vid_h_sync,
    input wire [TX_PIXELS_PER_CLOCK-1:0]                  tx_vid_de,
	
    input wire [(TX_VIDEO_BPS*3*TX_PIXELS_PER_CLOCK)-1:0] tx1_vid_data,
    input wire [TX_PIXELS_PER_CLOCK-1:0]                  tx1_vid_v_sync,
    input wire [TX_PIXELS_PER_CLOCK-1:0]                  tx1_vid_h_sync,
    input wire [TX_PIXELS_PER_CLOCK-1:0]                  tx1_vid_de,
	
    input wire [(TX_VIDEO_BPS*3*TX_PIXELS_PER_CLOCK)-1:0] tx2_vid_data,
    input wire [TX_PIXELS_PER_CLOCK-1:0]                  tx2_vid_v_sync,
    input wire [TX_PIXELS_PER_CLOCK-1:0]                  tx2_vid_h_sync,
    input wire [TX_PIXELS_PER_CLOCK-1:0]                  tx2_vid_de,
	
    input wire [(TX_VIDEO_BPS*3*TX_PIXELS_PER_CLOCK)-1:0] tx3_vid_data,
    input wire [TX_PIXELS_PER_CLOCK-1:0]                  tx3_vid_v_sync,
    input wire [TX_PIXELS_PER_CLOCK-1:0]                  tx3_vid_h_sync,
    input wire [TX_PIXELS_PER_CLOCK-1:0]                  tx3_vid_de,

    //************************************************************************
	// Video Image Interface
    //************************************************************************
	input wire												tx_im_clk,
	input wire [(TX_VIDEO_BPS*3*TX_PIXELS_PER_CLOCK)-1:0]	tx_im_data,
	input wire [TX_PIXELS_PER_CLOCK-1:0]					tx_im_valid,
	input wire												tx_im_sol,
	input wire												tx_im_eol,
	input wire												tx_im_sof,
	input wire												tx_im_eof,
	input wire												tx_im_locked,
	input wire												tx_im_interlace,
	input wire												tx_im_field,
	output wire												tx_im_full,
	output wire												tx_im_afull,
	output wire												tx_im_aempty,
	output wire												tx_im_empty,
	input wire												tx1_im_clk,
	input wire [(TX_VIDEO_BPS*3*TX_PIXELS_PER_CLOCK)-1:0]	tx1_im_data,
	input wire [TX_PIXELS_PER_CLOCK-1:0]					tx1_im_valid,
	input wire												tx1_im_sol,
	input wire												tx1_im_eol,
	input wire												tx1_im_sof,
	input wire												tx1_im_eof,
	input wire												tx1_im_locked,
	input wire												tx1_im_interlace,
	input wire												tx1_im_field,
	output wire												tx1_im_full,
	output wire												tx1_im_afull,
	output wire												tx1_im_aempty,
	output wire												tx1_im_empty,
	input wire												tx2_im_clk,
	input wire [(TX_VIDEO_BPS*3*TX_PIXELS_PER_CLOCK)-1:0]	tx2_im_data,
	input wire [TX_PIXELS_PER_CLOCK-1:0]					tx2_im_valid,
	input wire												tx2_im_sol,
	input wire												tx2_im_eol,
	input wire												tx2_im_sof,
	input wire												tx2_im_eof,
	input wire												tx2_im_locked,
	input wire												tx2_im_interlace,
	input wire												tx2_im_field,
	output wire												tx2_im_full,
	output wire												tx2_im_afull,
	output wire												tx2_im_aempty,
	output wire												tx2_im_empty,
	input wire												tx3_im_clk,
	input wire [(TX_VIDEO_BPS*3*TX_PIXELS_PER_CLOCK)-1:0]	tx3_im_data,
	input wire [TX_PIXELS_PER_CLOCK-1:0]					tx3_im_valid,
	input wire												tx3_im_sol,
	input wire												tx3_im_eol,
	input wire												tx3_im_sof,
	input wire												tx3_im_eof,
	input wire												tx3_im_locked,
	input wire												tx3_im_interlace,
	input wire												tx3_im_field,
	output wire												tx3_im_full,
	output wire												tx3_im_afull,
	output wire												tx3_im_aempty,
	output wire												tx3_im_empty,
	
    // RX video inputs have 4 duplicate interfaces for beta MST
    // support of 4 streams

    input wire rx_vid_clk,  // DP Sink Pixel clock frequency

    // RX Video output (rx_video_out)
    output wire [RX_PIXELS_PER_CLOCK-1:0]                  rx_vid_valid,
    output wire                                            rx_vid_sol,
    output wire                                            rx_vid_eol,
    output wire                                            rx_vid_sof,
    output wire                                            rx_vid_eof,
    output wire                                            rx_vid_locked,
    output wire                                            rx_vid_interlace,
    output wire                                            rx_vid_field,
    output wire [(RX_VIDEO_BPS*3*RX_PIXELS_PER_CLOCK)-1:0] rx_vid_data,

    input wire                                             rx1_vid_clk, // DP Sink Pixel clock frequency

    // RX Video output (rx_video_out)
    output wire [RX_PIXELS_PER_CLOCK-1:0]                  rx1_vid_valid,
    output wire                                            rx1_vid_sol,
    output wire                                            rx1_vid_eol,
    output wire                                            rx1_vid_sof,
    output wire                                            rx1_vid_eof,
    output wire                                            rx1_vid_locked,
    output wire                                            rx1_vid_interlace,
    output wire                                            rx1_vid_field,
    output wire [(RX_VIDEO_BPS*3*RX_PIXELS_PER_CLOCK)-1:0] rx1_vid_data,

    input wire                                             rx2_vid_clk, // DP Sink Pixel clock frequency

    // RX Video output (rx_video_out)
    output wire [RX_PIXELS_PER_CLOCK-1:0]                  rx2_vid_valid,
    output wire                                            rx2_vid_sol,
    output wire                                            rx2_vid_eol,
    output wire                                            rx2_vid_sof,
    output wire                                            rx2_vid_eof,
    output wire                                            rx2_vid_locked,
    output wire                                            rx2_vid_interlace,
    output wire                                            rx2_vid_field,
    output wire [(RX_VIDEO_BPS*3*RX_PIXELS_PER_CLOCK)-1:0] rx2_vid_data,

    input wire                                             rx3_vid_clk, // DP Sink Pixel clock frequency

    // RX Video output (rx_video_out)
    output wire [RX_PIXELS_PER_CLOCK-1:0]                  rx3_vid_valid,
    output wire                                            rx3_vid_sol,
    output wire                                            rx3_vid_eol,
    output wire                                            rx3_vid_sof,
    output wire                                            rx3_vid_eof,
    output wire                                            rx3_vid_locked,
    output wire                                            rx3_vid_interlace,
    output wire                                            rx3_vid_field,
    output wire [(RX_VIDEO_BPS*3*RX_PIXELS_PER_CLOCK)-1:0] rx3_vid_data,

    //************************************************************************
    // AUX clock domain
    //
    // Clock is aux_clk, reset is aux_reset
    //************************************************************************
    input wire aux_reset,
    input wire aux_clk, // 16 MHz

    // TX AUX channel physical signals (tx_aux)
    input  wire tx_aux_in,
    output wire tx_aux_out, 
    output wire tx_aux_oe,
    input  wire tx_hpd,

    // TX AUX Debug (AV-ST) sinks are designed to interface to Nios 32-bit bus via DC FIFO (tx_aux_debug)
    output wire [31:0] tx_aux_debug_data,
    output wire        tx_aux_debug_valid,
    output wire        tx_aux_debug_sop, 
    output wire        tx_aux_debug_eop, 
    output wire        tx_aux_debug_err,
    output wire        tx_aux_debug_cha,

    // RX AUX channel physical signals (rx_aux)
    input  wire rx_aux_in,
    output wire rx_aux_out, 
    output wire rx_aux_oe,
    output wire rx_hpd,
    input  wire rx_cable_detect,
    input  wire rx_pwr_detect,

    // RX AUX Debug (AV-ST) sinks are designed to interface to Nios 32-bit bus via DC FIFO (rx_aux_debug)
    output wire [31:0] rx_aux_debug_data,
    output wire        rx_aux_debug_valid,
    output wire        rx_aux_debug_sop, 
    output wire        rx_aux_debug_eop, 
    output wire        rx_aux_debug_err,
    output wire        rx_aux_debug_cha,

    // RX EDID ROM interface (AV-MM) rx_edid
    output wire [7:0] rx_edid_address,
    output wire       rx_edid_read,
    output wire       rx_edid_write,
    output wire [7:0] rx_edid_writedata,
    input  wire [7:0] rx_edid_readdata,
    input  wire       rx_edid_waitrequest,

    //************************************************************************
    // GTC interface (Global Time Code)
    //************************************************************************
    input wire tx_clk_gtc,
    input wire [31:0] tx_gtc_value,
    output wire [31:0] tx_gtc_fce,
    output wire [31:0] tx_gtc_remote,
    output wire tx_gtc_update,

    input wire rx_clk_gtc,
    input wire [31:0] rx_gtc_value,
    output wire [31:0] rx_gtc_fce,
    output wire [31:0] rx_gtc_remote,
    output wire rx_gtc_update,

    //************************************************************************
    // Debug domain (RX link status data)
    //
    // Clock and reset are a mixture, should be used for debug not design
    //************************************************************************
    output wire [4:0] rx_lane_count,

    // Top level RX post scrambler signals & recovered clock for each
    // stream
    output wire [4*8*RX_SYMBOLS_PER_CLOCK-1:0] rx_stream_data,
    output wire [4*1*RX_SYMBOLS_PER_CLOCK-1:0] rx_stream_ctrl,
    output wire	                               rx_stream_valid,
    output wire                                rx_stream_clk,

    output wire [4*8*RX_SYMBOLS_PER_CLOCK-1:0] rx1_stream_data,
    output wire [4*1*RX_SYMBOLS_PER_CLOCK-1:0] rx1_stream_ctrl,
    output wire                                rx1_stream_valid,
    output wire                                rx1_stream_clk,

    output wire [4*8*RX_SYMBOLS_PER_CLOCK-1:0] rx2_stream_data,
    output wire [4*1*RX_SYMBOLS_PER_CLOCK-1:0] rx2_stream_ctrl,
    output wire                                rx2_stream_valid,
    output wire                                rx2_stream_clk,

    output wire [4*8*RX_SYMBOLS_PER_CLOCK-1:0] rx3_stream_data,
    output wire [4*1*RX_SYMBOLS_PER_CLOCK-1:0] rx3_stream_ctrl,
    output wire                                rx3_stream_valid,
    output wire                                rx3_stream_clk,

    //************************************************************************
    // Secondary data domain 
    //
    // Clock is mainly TX/RX XCVR recovered clock, but some domains have 
    // their own clock. Reset is reset
    //************************************************************************
    output wire tx_ss_clk,  // 81 MHz or 135 MHz
    output wire rx_ss_clk,  // 81 MHz or 135 MHz

    // MSA Domain per stream
    output wire [216:0] rx_msa,
    output wire [216:0] rx1_msa,
    output wire [216:0] rx2_msa,
    output wire [216:0] rx3_msa,

    // SS Domain

    // TX SS Data (AV-ST) (tx_ss)
    input  wire [127:0] tx_ss_data,
    input  wire         tx_ss_valid,
    output wire         tx_ss_ready,
    input  wire         tx_ss_sop,
    input  wire         tx_ss_eop,

    input  wire [127:0] tx1_ss_data,
    input  wire         tx1_ss_valid,
    output wire         tx1_ss_ready,
    input  wire         tx1_ss_sop,
    input  wire         tx1_ss_eop,

    input  wire [127:0] tx2_ss_data,
    input  wire         tx2_ss_valid,
    output wire         tx2_ss_ready,
    input  wire         tx2_ss_sop,
    input  wire         tx2_ss_eop,

    input  wire [127:0] tx3_ss_data,
    input  wire         tx3_ss_valid,
    output wire         tx3_ss_ready,
    input  wire         tx3_ss_sop,
    input  wire         tx3_ss_eop,

    // RX SS Data (AV-ST) (rx_ss) one per stream
    output wire [159:0] rx_ss_data,
    output wire         rx_ss_valid,
    output wire         rx_ss_sop,
    output wire         rx_ss_eop,

    output wire [159:0] rx1_ss_data,
    output wire         rx1_ss_valid,
    output wire         rx1_ss_sop,
    output wire         rx1_ss_eop,

    output wire [159:0] rx2_ss_data,
    output wire         rx2_ss_valid,
    output wire         rx2_ss_sop,
    output wire         rx2_ss_eop,

    output wire [159:0] rx3_ss_data,
    output wire         rx3_ss_valid,
    output wire         rx3_ss_sop,
    output wire         rx3_ss_eop,

    // Audio Domain
    // TX Audio data (tx_audio)
    input wire                         tx_audio_clk, // 44 KHz
    input wire [TX_AUDIO_CHANS*32-1:0] tx_audio_lpcm_data,
    input wire                         tx_audio_valid,
    input wire                         tx_audio_mute,

    input wire                         tx1_audio_clk, // 44 KHz
    input wire [TX_AUDIO_CHANS*32-1:0] tx1_audio_lpcm_data,
    input wire                         tx1_audio_valid,
    input wire                         tx1_audio_mute,

    input wire                         tx2_audio_clk, // 44 KHz
    input wire [TX_AUDIO_CHANS*32-1:0] tx2_audio_lpcm_data,
    input wire                         tx2_audio_valid,
    input wire                         tx2_audio_mute,

    input wire                         tx3_audio_clk, // 44 KHz
    input wire [TX_AUDIO_CHANS*32-1:0] tx3_audio_lpcm_data,
    input wire                         tx3_audio_valid,
    input wire                         tx3_audio_mute,

    // RX Audio data (rx_audio) per stream
    output wire [RX_AUDIO_CHANS*32-1:0]	rx_audio_lpcm_data,
    output wire                         rx_audio_valid,
    output wire                         rx_audio_mute,
    output wire [39:0]                  rx_audio_infoframe,

    output wire [RX_AUDIO_CHANS*32-1:0] rx1_audio_lpcm_data,
    output wire                         rx1_audio_valid,
    output wire                         rx1_audio_mute,
    output wire [39:0]                  rx1_audio_infoframe,

    output wire [RX_AUDIO_CHANS*32-1:0] rx2_audio_lpcm_data,
    output wire                         rx2_audio_valid,
    output wire                         rx2_audio_mute,
    output wire [39:0]                  rx2_audio_infoframe,

    output wire [RX_AUDIO_CHANS*32-1:0] rx3_audio_lpcm_data,
    output wire                         rx3_audio_valid,
    output wire                         rx3_audio_mute,
    output wire [39:0]                  rx3_audio_infoframe
);

    // Wires connecting DP to PHY IP
    // XCVR data interface can now be 2 symbols or 4 symbols (20-bit
    // or 40-bit wide)
    wire [(1 + TX_MAX_LANE_COUNT + TX_MAX_LANE_COUNT*10*TX_SYMBOLS_PER_CLOCK)-1:0] conduit_to_gxb_tx;
    wire [(TX_MAX_LANE_COUNT + 1)-1:0]                                             conduit_from_gxb_tx;

    // XCVR data interface can now be 2 symbols or 4 symbols (20-bit
    // or 40-bit wide)
    wire [(1 + RX_MAX_LANE_COUNT + RX_MAX_LANE_COUNT + RX_MAX_LANE_COUNT + 1 + RX_MAX_LANE_COUNT + RX_MAX_LANE_COUNT) -1:0] conduit_to_gxb_rx;
    wire [(RX_MAX_LANE_COUNT + RX_MAX_LANE_COUNT + RX_MAX_LANE_COUNT + 10*RX_SYMBOLS_PER_CLOCK*RX_MAX_LANE_COUNT)-1:0]  conduit_from_gxb_rx;

    // emp_lane3..emp_lane0, vod_lane3..vod_lane0, tx_link_rate,
    // reconfig_link_rate_req, reconfig_analog_req
    wire [(2*TX_MAX_LANE_COUNT + 2*TX_MAX_LANE_COUNT + 8 + 2)-1:0] conduit_to_tx_reconfig;
    wire [0:0]                                                     conduit_from_tx_reconfig;

    // rx_link_rate, reconfig_link_rate_req
    wire [(2*RX_MAX_LANE_COUNT + 2*RX_MAX_LANE_COUNT + 1 + 8 + 1)-1:0] conduit_to_rx_reconfig;
    wire [0:0]         conduit_from_rx_reconfig;

    // Wires connecting to the TX PHY IP to DP 
    wire [TX_MAX_LANE_COUNT-1:0]                           btc_tx_digitalreset;

    // Wires connecting to the RX PHY IP to DP
    wire [RX_MAX_LANE_COUNT-1:0]                           btc_rx_analogreset;
    wire [RX_MAX_LANE_COUNT-1:0]                           btc_rx_digitalreset;

    // Wires connecting to the TX Reconfig mgmt logic to DP
    wire                             btc_tx_analog_reconfig_req;
    wire [7:0]                       btc_tx_link_rate;
    wire [(TX_MAX_LANE_COUNT*2)-1:0] btc_tx_vod;
    wire [(TX_MAX_LANE_COUNT*2)-1:0] btc_tx_emp; 

    // Wires connecting to the RX Reconfig mgmt logic to DP
    wire [7:0] btc_rx_link_rate;

    bitec_dp #(
       // Pass paramters
       .TX_MAX_LINK_RATE(TX_MAX_LINK_RATE),
       .TX_MAX_LANE_COUNT(TX_MAX_LANE_COUNT),
       .TX_SCRAMBLER_SEED(TX_SCRAMBLER_SEED),
       .TX_SUPPORT_SS(TX_SUPPORT_SS),
       .TX_PIXELS_PER_CLOCK(TX_PIXELS_PER_CLOCK),  // replaces TX_ODD_EVEN_PIXEL
       .TX_SYMBOLS_PER_CLOCK(TX_SYMBOLS_PER_CLOCK),
	   .TX_VIDEO_IM_ENABLE(TX_VIDEO_IM_ENABLE),
       .TX_SUPPORT_18BPP(TX_SUPPORT_18BPP),
       .TX_SUPPORT_24BPP(TX_SUPPORT_24BPP),
       .TX_SUPPORT_30BPP(TX_SUPPORT_30BPP),
       .TX_SUPPORT_36BPP(TX_SUPPORT_36BPP),
       .TX_SUPPORT_48BPP(TX_SUPPORT_48BPP),
       .TX_SUPPORT_YCBCR422_16BPP(TX_SUPPORT_YCBCR422_16BPP),
       .TX_SUPPORT_YCBCR422_20BPP(TX_SUPPORT_YCBCR422_20BPP),
       .TX_SUPPORT_YCBCR422_24BPP(TX_SUPPORT_YCBCR422_24BPP),
       .TX_SUPPORT_YCBCR422_32BPP(TX_SUPPORT_YCBCR422_32BPP),
       .TX_SUPPORT_YCBCR420_12BPP(TX_SUPPORT_YCBCR420_12BPP),
       .TX_SUPPORT_YCBCR420_15BPP(TX_SUPPORT_YCBCR420_15BPP),
       .TX_SUPPORT_YCBCR420_18BPP(TX_SUPPORT_YCBCR420_18BPP),
       .TX_SUPPORT_YCBCR420_24BPP(TX_SUPPORT_YCBCR420_24BPP),
       .TX_SUPPORT_DP(TX_SUPPORT_DP),
       .TX_SUPPORT_HDCP1(0),
       .TX_SUPPORT_HDCP2(0),
       .TX_VIDEO_BPC(TX_VIDEO_BPS),  // renamed in 40-bit code
       .TX_SUPPORT_AUTOMATED_TEST(TX_SUPPORT_AUTOMATED_TEST),
       .TX_AUX_DEBUG(TX_AUX_DEBUG),

       .RX_MAX_LINK_RATE(RX_MAX_LINK_RATE),
       .RX_MAX_LANE_COUNT(RX_MAX_LANE_COUNT),
       .RX_SCRAMBLER_SEED(RX_SCRAMBLER_SEED),
       .RX_SUPPORT_EDP(RX_SUPPORT_EDP),
       .RX_SUPPORT_SS(RX_SUPPORT_SS),
       .RX_SUPPORT_I2CMASTER(0),
       .RX_I2C_SCL_KHZ(100),
       .RX_SUPPORT_LPM(0),
       .RX_PIXELS_PER_CLOCK(RX_PIXELS_PER_CLOCK),  // replaces RX_ODD_EVEN_PIXEL
       .RX_SYMBOLS_PER_CLOCK(RX_SYMBOLS_PER_CLOCK),
       .RX_SUPPORT_AUTOMATED_TEST(RX_SUPPORT_AUTOMATED_TEST),

       .RX_SUPPORT_18BPP(RX_SUPPORT_18BPP),
       .RX_SUPPORT_24BPP(RX_SUPPORT_24BPP),
       .RX_SUPPORT_30BPP(RX_SUPPORT_30BPP),
       .RX_SUPPORT_36BPP(RX_SUPPORT_36BPP),
       .RX_SUPPORT_48BPP(RX_SUPPORT_48BPP),
       .RX_SUPPORT_YCBCR422_16BPP(RX_SUPPORT_YCBCR422_16BPP),
       .RX_SUPPORT_YCBCR422_20BPP(RX_SUPPORT_YCBCR422_20BPP),
       .RX_SUPPORT_YCBCR422_24BPP(RX_SUPPORT_YCBCR422_24BPP),
       .RX_SUPPORT_YCBCR422_32BPP(RX_SUPPORT_YCBCR422_32BPP),
       .RX_SUPPORT_YCBCR420_12BPP(RX_SUPPORT_YCBCR420_12BPP),
       .RX_SUPPORT_YCBCR420_15BPP(RX_SUPPORT_YCBCR420_15BPP),
       .RX_SUPPORT_YCBCR420_18BPP(RX_SUPPORT_YCBCR420_18BPP),
       .RX_SUPPORT_YCBCR420_24BPP(RX_SUPPORT_YCBCR420_24BPP),
       .RX_SUPPORT_DP(RX_SUPPORT_DP),
       .RX_SUPPORT_HDCP1(0),
       .RX_SUPPORT_HDCP2(0),
       .RX_VIDEO_BPC(RX_VIDEO_BPS),  // renamed in 40-bit code

       .RX_AUX_DEBUG(RX_AUX_DEBUG),
       .RX_IEEE_OUI(RX_IEEE_OUI),
       .RX_AUX_GPU(RX_AUX_GPU),
       .RX_EXPORT_MSA(RX_EXPORT_MSA),

       .TX_SUPPORT_AUDIO(TX_SUPPORT_AUDIO),
       .TX_AUDIO_CHANS(TX_AUDIO_CHANS),
       .RX_SUPPORT_AUDIO(RX_SUPPORT_AUDIO),
       .RX_AUDIO_CHANS(RX_AUDIO_CHANS),

       .TX_SUPPORT_MST            (TX_SUPPORT_MST),
       .TX_MAX_NUM_OF_STREAMS     (TX_MAX_NUM_OF_STREAMS),
       .RX_SUPPORT_MST            (RX_SUPPORT_MST),
       .RX_MAX_NUM_OF_STREAMS     (RX_MAX_NUM_OF_STREAMS),

       .TX_SUPPORT_GTC            (TX_SUPPORT_GTC),
       .RX_SUPPORT_GTC            (RX_SUPPORT_GTC)
    ) bitec_dp_inst (
       // Common signals
       .reset_av (reset), 
       .clk_av   (clk), 
       .clk_16   (aux_clk),
       .reset_16 (aux_reset),
       .clk_cal  (clk_cal),

       .conduit_to_gxb_tx (conduit_to_gxb_tx),
       .conduit_from_gxb_tx (conduit_from_gxb_tx),

       .conduit_to_gxb_rx (conduit_to_gxb_rx),
       .conduit_from_gxb_rx (conduit_from_gxb_rx),

       // Bitec now spliting the reconfig conduits
       .conduit_to_tx_reconfig (conduit_to_tx_reconfig),
       .conduit_from_tx_reconfig (conduit_from_tx_reconfig),
       .conduit_to_rx_reconfig (conduit_to_rx_reconfig),
       .conduit_from_rx_reconfig (conduit_from_rx_reconfig),

       // Input video interface
       .tx0_vid_vsync             (tx_vid_v_sync),
       .tx0_vid_hsync             (tx_vid_h_sync),
       .tx0_vid_data              (tx_vid_data),
       .tx0_vid_de                (tx_vid_de),   
       .tx0_vid_clk               (tx_vid_clk),
       .tx1_vid_clk               (tx1_vid_clk),
       .tx1_vid_data              (tx1_vid_data),
       .tx1_vid_vsync             (tx1_vid_v_sync),
       .tx1_vid_hsync             (tx1_vid_h_sync),
       .tx1_vid_de                (tx1_vid_de),
       .tx2_vid_clk               (tx2_vid_clk),
       .tx2_vid_data              (tx2_vid_data),
       .tx2_vid_vsync             (tx2_vid_v_sync),
       .tx2_vid_hsync             (tx2_vid_h_sync),
       .tx2_vid_de                (tx2_vid_de),
       .tx3_vid_clk               (tx3_vid_clk),
       .tx3_vid_data              (tx3_vid_data),
       .tx3_vid_vsync             (tx3_vid_v_sync),
       .tx3_vid_hsync             (tx3_vid_h_sync),
       .tx3_vid_de                (tx3_vid_de),

	   // Input Video Image Interface
       .tx0_im_clk				(tx_im_clk),
       .tx0_im_data				(tx_im_data),
       .tx0_im_valid			(tx_im_valid),
       .tx0_im_sol				(tx_im_sol),
       .tx0_im_eol				(tx_im_eol),
       .tx0_im_sof				(tx_im_sof),
       .tx0_im_eof				(tx_im_eof),
       .tx0_im_locked			(tx_im_locked),
       .tx0_im_interlace		(tx_im_interlace),
       .tx0_im_field			(tx_im_field),
       .tx0_im_full				(tx_im_full),
       .tx0_im_afull			(tx_im_afull),
       .tx0_im_aempty			(tx_im_aempty),
       .tx0_im_empty			(tx_im_empty),
       .tx1_im_clk				(tx1_im_clk),
       .tx1_im_data				(tx1_im_data),
       .tx1_im_valid			(tx1_im_valid),
       .tx1_im_sol				(tx1_im_sol),
       .tx1_im_eol				(tx1_im_eol),
       .tx1_im_sof				(tx1_im_sof),
       .tx1_im_eof				(tx1_im_eof),
       .tx1_im_locked			(tx1_im_locked),
       .tx1_im_interlace		(tx1_im_interlace),
       .tx1_im_field			(tx1_im_field),
       .tx1_im_full				(tx1_im_full),
       .tx1_im_afull			(tx1_im_afull),
       .tx1_im_aempty			(tx1_im_aempty),
       .tx1_im_empty			(tx1_im_empty),
       .tx2_im_clk				(tx2_im_clk),
       .tx2_im_data				(tx2_im_data),
       .tx2_im_valid			(tx2_im_valid),
       .tx2_im_sol				(tx2_im_sol),
       .tx2_im_eol				(tx2_im_eol),
       .tx2_im_sof				(tx2_im_sof),
       .tx2_im_eof				(tx2_im_eof),
       .tx2_im_locked			(tx2_im_locked),
       .tx2_im_interlace		(tx2_im_interlace),
       .tx2_im_field			(tx2_im_field),
       .tx2_im_full				(tx2_im_full),
       .tx2_im_afull			(tx2_im_afull),
       .tx2_im_aempty			(tx2_im_aempty),
       .tx2_im_empty			(tx2_im_empty),
       .tx3_im_clk				(tx3_im_clk),
       .tx3_im_data				(tx3_im_data),
       .tx3_im_valid			(tx3_im_valid),
       .tx3_im_sol				(tx3_im_sol),
       .tx3_im_eol				(tx3_im_eol),
       .tx3_im_sof				(tx3_im_sof),
       .tx3_im_eof				(tx3_im_eof),
       .tx3_im_locked			(tx3_im_locked),
       .tx3_im_interlace		(tx3_im_interlace),
       .tx3_im_field			(tx3_im_field),
       .tx3_im_full				(tx3_im_full),
       .tx3_im_afull			(tx3_im_afull),
       .tx3_im_aempty			(tx3_im_aempty),
       .tx3_im_empty			(tx3_im_empty),	   

       // TX AUX channel physical signals
       .tx_aux_in  (tx_aux_in),
       .tx_aux_out (tx_aux_out), 
       .tx_aux_oe  (tx_aux_oe),
       .tx_hpd     (tx_hpd),

       // RX AUX channel physical signals
       .rx_aux_in       (rx_aux_in),
       .rx_aux_out      (rx_aux_out), 
       .rx_aux_oe       (rx_aux_oe),
       .rx_hpd          (rx_hpd),
       .rx_cable_detect (rx_cable_detect),
       .rx_pwr_detect   (rx_pwr_detect),

       // RX I2C Master interfaces
       .rx_i2c0_scl(),
       .rx_i2c0_sda(),
       .rx_i2c1_scl(),
       .rx_i2c1_sda(),

       // TX Avalon Slave
       .av_tx_control_address      (tx_mgmt_address),
       .av_tx_control_chipselect   (tx_mgmt_chipselect),
       .av_tx_control_read         (tx_mgmt_read),
       .av_tx_control_write        (tx_mgmt_write),
       .av_tx_control_writedata    (tx_mgmt_writedata),
       .av_tx_control_readdata     (tx_mgmt_readdata),
       .av_tx_control_waitrequest  (tx_mgmt_waitrequest),
       .av_tx_control_irq          (tx_mgmt_irq),

       // RX Avalon slave
       .av_rx_control_chipselect   (rx_mgmt_chipselect),
       .av_rx_control_read         (rx_mgmt_read),
       .av_rx_control_write        (rx_mgmt_write),
       .av_rx_control_address      (rx_mgmt_address),
       .av_rx_control_writedata    (rx_mgmt_writedata),
       .av_rx_control_readdata     (rx_mgmt_readdata),
       .av_rx_control_waitrequest  (rx_mgmt_waitrequest),
       .av_rx_control_irq          (rx_mgmt_irq),

       // RX AUX debug stream
       .aux_rx_debug_st_data  (rx_aux_debug_data),
       .aux_rx_debug_st_valid (rx_aux_debug_valid),
       .aux_rx_debug_st_sop   (rx_aux_debug_sop),
       .aux_rx_debug_st_eop   (rx_aux_debug_eop), 
       .aux_rx_debug_st_err   (rx_aux_debug_err),
       .aux_rx_debug_st_cha   (rx_aux_debug_cha),
       // .aux_rx_debug_st_emp (), // Unnecessary since 11.1

       // TX AUX debug stream
       .aux_tx_debug_st_data  (tx_aux_debug_data),
       .aux_tx_debug_st_valid (tx_aux_debug_valid),
       .aux_tx_debug_st_sop   (tx_aux_debug_sop),
       .aux_tx_debug_st_eop   (tx_aux_debug_eop), 
       .aux_tx_debug_st_err   (tx_aux_debug_err),
       .aux_tx_debug_st_cha   (tx_aux_debug_cha),
       // .aux_tx_debug_st_emp (), // Unnecessary since 11.1

       // RX Status signals
       .rx_lane_count (rx_lane_count),

       // TX Secondary Stream data
       .tx_ss_clk                 (tx_ss_clk),
       .tx0_ss_data               (tx_ss_data),
       .tx0_ss_valid              (tx_ss_valid),
       .tx0_ss_ready              (tx_ss_ready),
       .tx0_ss_sop                (tx_ss_sop),
       .tx0_ss_eop                (tx_ss_eop),
       .tx1_ss_ready              (tx1_ss_ready),
       .tx1_ss_valid              (tx1_ss_valid),
       .tx1_ss_data               (tx1_ss_data),
       .tx1_ss_sop                (tx1_ss_sop), 
       .tx1_ss_eop                (tx1_ss_eop),
       .tx2_ss_ready              (tx2_ss_ready),
       .tx2_ss_valid              (tx2_ss_valid),
       .tx2_ss_data               (tx2_ss_data),
       .tx2_ss_sop                (tx2_ss_sop),
       .tx2_ss_eop                (tx2_ss_eop),
       .tx3_ss_ready              (tx3_ss_ready),
       .tx3_ss_valid              (tx3_ss_valid),
       .tx3_ss_data               (tx3_ss_data),
       .tx3_ss_sop                (tx3_ss_sop),
       .tx3_ss_eop                (tx3_ss_eop),

       // RX external EDID
       .rx_edid_addr        (rx_edid_address),
       .rx_edid_readdata    (rx_edid_readdata),
       .rx_edid_writedata   (rx_edid_writedata),
       .rx_edid_ren         (rx_edid_read),
       .rx_edid_wen         (rx_edid_write),
       .rx_edid_waitrequest (rx_edid_waitrequest),

       // TX Audio data
       .tx0_audio_clk             (tx_audio_clk),
       .tx0_audio_lpcm_data       (tx_audio_lpcm_data),
       .tx0_audio_valid           (tx_audio_valid),
       .tx0_audio_mute            (tx_audio_mute),
       .tx1_audio_clk             (tx1_audio_clk),
       .tx1_audio_lpcm_data       (tx1_audio_lpcm_data),
       .tx1_audio_valid           (tx1_audio_valid),
       .tx1_audio_mute            (tx1_audio_mute),
       .tx2_audio_clk             (tx2_audio_clk),
       .tx2_audio_lpcm_data       (tx2_audio_lpcm_data),
       .tx2_audio_valid           (tx2_audio_valid),
       .tx2_audio_mute            (tx2_audio_mute),
       .tx3_audio_clk             (tx3_audio_clk),
       .tx3_audio_lpcm_data       (tx3_audio_lpcm_data),
       .tx3_audio_valid           (tx3_audio_valid),
       .tx3_audio_mute            (tx3_audio_mute),

        // HDCP Encoder and Decoder
       .to_hdcp_dec    (),
       .from_hdcp_dec  ({RX_SYMBOLS_PER_CLOCK*RX_MAX_LANE_COUNT*9+8+32+1+1{1'b0}}),
       .to_hdcp_enc    (),
       .from_hdcp_enc  ({TX_SYMBOLS_PER_CLOCK*TX_MAX_LANE_COUNT*9+32+1+1{1'b0}}),

       // ---- RX Stream specific signals -----
  
       // Output Video Interface
       .rx0_im_valid      (rx_vid_valid),
       .rx0_im_sol        (rx_vid_sol),
       .rx0_im_eol        (rx_vid_eol),
       .rx0_im_sof        (rx_vid_sof),
       .rx0_im_eof        (rx_vid_eof),
       .rx0_im_locked     (rx_vid_locked),
       .rx0_im_overflow   (rx_vid_overflow),
       .rx0_im_interlace  (rx_vid_interlace),
       .rx0_im_field      (rx_vid_field),
       .rx0_im_clk        (rx_vid_clk),
       .rx0_im_data       (rx_vid_data),
       .rx1_im_valid      (rx1_vid_valid),
       .rx1_im_sol        (rx1_vid_sol),
       .rx1_im_eol        (rx1_vid_eol),
       .rx1_im_sof        (rx1_vid_sof),
       .rx1_im_eof        (rx1_vid_eof),
       .rx1_im_locked     (rx1_vid_locked),
       .rx1_im_overflow   (rx1_vid_overflow),
       .rx1_im_interlace  (rx1_vid_interlace),
       .rx1_im_field      (rx1_vid_field),
       .rx1_im_clk        (rx1_vid_clk),
       .rx1_im_data       (rx1_vid_data),
       .rx2_im_valid      (rx2_vid_valid),
       .rx2_im_sol        (rx2_vid_sol),
       .rx2_im_eol        (rx2_vid_eol),
       .rx2_im_sof        (rx2_vid_sof),
       .rx2_im_eof        (rx2_vid_eof),
       .rx2_im_locked     (rx2_vid_locked),
       .rx2_im_overflow   (rx2_vid_overflow),
       .rx2_im_interlace  (rx2_vid_interlace),
       .rx2_im_field      (rx2_vid_field),
       .rx2_im_clk        (rx2_vid_clk),
       .rx2_im_data       (rx2_vid_data),
       .rx3_im_valid      (rx3_vid_valid),
       .rx3_im_sol        (rx3_vid_sol),
       .rx3_im_eol        (rx3_vid_eol),
       .rx3_im_sof        (rx3_vid_sof),
       .rx3_im_eof        (rx3_vid_eof),
       .rx3_im_locked     (rx3_vid_locked),
       .rx3_im_overflow   (rx3_vid_overflow),
       .rx3_im_interlace  (rx3_vid_interlace),
       .rx3_im_field      (rx3_vid_field),
       .rx3_im_clk        (rx3_vid_clk),
       .rx3_im_data       (rx3_vid_data),

        // Post scrambler signals
        .rx0_stream_data(rx_stream_data),
        .rx0_stream_ctrl(rx_stream_ctrl),
        .rx0_stream_valid(rx_stream_valid),
        .rx0_stream_clk(rx_stream_clk),
        .rx1_stream_data(rx1_stream_data),
        .rx1_stream_ctrl(rx1_stream_ctrl),
        .rx1_stream_valid(rx1_stream_valid),
        .rx1_stream_clk(rx1_stream_clk),
        .rx2_stream_data(rx2_stream_data),
        .rx2_stream_ctrl(rx2_stream_ctrl),
        .rx2_stream_valid(rx2_stream_valid),
        .rx2_stream_clk(rx2_stream_clk),
        .rx3_stream_data(rx3_stream_data),
        .rx3_stream_ctrl(rx3_stream_ctrl),
        .rx3_stream_valid(rx3_stream_valid),
        .rx3_stream_clk(rx3_stream_clk),

        // Secondary Stream data
        .rx_ss_clk(rx_ss_clk),
        .rx0_ss_data(rx_ss_data),
        .rx0_ss_valid(rx_ss_valid),
        .rx0_ss_sop(rx_ss_sop),
        .rx0_ss_eop(rx_ss_eop),
        .rx1_ss_data(rx1_ss_data),
        .rx1_ss_valid(rx1_ss_valid),
        .rx1_ss_sop(rx1_ss_sop),
        .rx1_ss_eop(rx1_ss_eop),
        .rx2_ss_data(rx2_ss_data),
        .rx2_ss_valid(rx2_ss_valid),
        .rx2_ss_sop(rx2_ss_sop),
        .rx2_ss_eop(rx2_ss_eop),
        .rx3_ss_data(rx3_ss_data),
        .rx3_ss_valid(rx3_ss_valid),
        .rx3_ss_sop(rx3_ss_sop),
        .rx3_ss_eop(rx3_ss_eop),

        // MSA data
        .rx0_msa(rx_msa), 
        .rx1_msa(rx1_msa), 
        .rx2_msa(rx2_msa), 
        .rx3_msa(rx3_msa), 

        // Audio data
       .rx0_audio_clk(),                         // ignore, same as rx_ss_clk
       .rx0_audio_lpcm_data(rx_audio_lpcm_data),
       .rx0_audio_valid(rx_audio_valid), 
       .rx0_audio_mute(rx_audio_mute),
       .rx0_audio_infoframe(rx_audio_infoframe),
       .rx1_audio_clk(),                         // ignore, same as rx_ss_clk
       .rx1_audio_lpcm_data(rx1_audio_lpcm_data),
       .rx1_audio_valid(rx1_audio_valid), 
       .rx1_audio_mute(rx1_audio_mute),
       .rx1_audio_infoframe(rx1_audio_infoframe),
       .rx2_audio_clk(),                         // ignore, same as rx_ss_clk
       .rx2_audio_lpcm_data(rx2_audio_lpcm_data),
       .rx2_audio_valid(rx2_audio_valid), 
       .rx2_audio_mute(rx2_audio_mute),
       .rx2_audio_infoframe(rx2_audio_infoframe),
       .rx3_audio_clk(),                         // ignore, same as rx_ss_clk
       .rx3_audio_lpcm_data(rx3_audio_lpcm_data),
       .rx3_audio_valid(rx3_audio_valid), 
       .rx3_audio_mute(rx3_audio_mute),
       .rx3_audio_infoframe(rx3_audio_infoframe),

        // LPM
       .clk_ser                   (1'b0),                                                                                                                                                                                                      //             (terminated)
       .lpm_remregs_miso          (),                                                                                                                                                                                                      //             (terminated)
       .lpm_remregs_mosi          (1'b0),                                                                                                                                                                                                  //             (terminated)
       .lpm_remregs_sclk          (1'b0),                                                                                                                                                                                                  //             (terminated)
       .lpm_remregs_ssn           (1'b0),       
       .lpm_irq                   (),
       .lpm_pwrdwn_req            (1'b0),
       .lpm_pwrdwn_ack            (),

       // GTC
       .tx_clk_gtc                (tx_clk_gtc),
       .tx_gtc_value              (tx_gtc_value),
       .tx_gtc_fce                (tx_gtc_fce),
       .tx_gtc_remote             (tx_gtc_remote),
       .tx_gtc_update             (tx_gtc_update),
       .rx_clk_gtc                (rx_clk_gtc),
       .rx_gtc_value              (rx_gtc_value),
       .rx_gtc_fce                (rx_gtc_fce),
       .rx_gtc_remote             (rx_gtc_remote),
       .rx_gtc_update             (rx_gtc_update),
       
       // APB Control
       .apb_tx_control_paddr        (9'd0),
       .apb_tx_control_psel         (1'b0),
       .apb_tx_control_penable      (1'b0),
       .apb_tx_control_pwrite       (1'b0),
       .apb_tx_control_pwdata       (32'd0),
       .apb_tx_control_prdata       (),
       .apb_tx_control_pready       (),
       .apb_tx_control_irq          (),

       .apb_rx_control_paddr        (9'd0),
       .apb_rx_control_psel         (1'b0),
       .apb_rx_control_penable      (1'b0),
       .apb_rx_control_pwrite       (1'b0),
       .apb_rx_control_pwdata       (32'd0),
       .apb_rx_control_prdata       (),
       .apb_rx_control_pready       (),
       .apb_rx_control_irq          ()  
    );

    //***************************************************************************
    // Adaptation logic to connect the Bitec DP core to the XCVR PHY datapaths
    //***************************************************************************
    assign {tx_pll_powerdown, btc_tx_digitalreset, tx_parallel_data} = conduit_to_gxb_tx;

    // Reset the TX PLL and channel when doing a reconfiguration
    assign tx_analogreset    = {TX_MAX_LANE_COUNT{tx_pll_powerdown | tx_reconfig_busy}};
    assign tx_digitalreset   = btc_tx_digitalreset | {TX_MAX_LANE_COUNT{tx_reconfig_busy}};

    assign conduit_from_gxb_tx = {tx_std_clkout, tx_pll_locked};

    assign {rx_restart, rx_bitslip, rx_set_locktodata, rx_set_locktoref} = conduit_to_gxb_rx[5*RX_MAX_LANE_COUNT + 1:2*RX_MAX_LANE_COUNT+1];
    assign {btc_rx_analogreset, btc_rx_digitalreset} = conduit_to_gxb_rx[2*RX_MAX_LANE_COUNT-1:0];


    // Reset the RX PLL and channel when doing a reconfiguration
    assign rx_analogreset   = btc_rx_analogreset | {RX_MAX_LANE_COUNT{rx_reconfig_busy}};
    assign rx_digitalreset  = btc_rx_digitalreset | {RX_MAX_LANE_COUNT{rx_reconfig_busy}};

    assign conduit_from_gxb_rx = {rx_std_clkout, rx_is_lockedtodata, rx_is_lockedtoref, rx_parallel_data};

    //***************************************************************************
    // Adaptation logic to connect the Bitec DP core to the XCVR
    // reconfig management logic
    //***************************************************************************

    // TX: Reconfig <- DP
    assign {btc_tx_emp, btc_tx_vod, btc_tx_link_rate} = conduit_to_tx_reconfig[(2*TX_MAX_LANE_COUNT + 2*TX_MAX_LANE_COUNT + 8 + 2)-1:2];
    assign btc_tx_analog_reconfig_req = conduit_to_tx_reconfig[0];


    // TX: Reconfig -> DP
    assign conduit_from_tx_reconfig = | tx_cal_busy;

    // RX: Reconfig <- DP
    assign btc_rx_link_rate = conduit_to_rx_reconfig[8:1];

    // RX: Reconfig -> DP
    assign conduit_from_rx_reconfig = | rx_cal_busy;
	
	assign tx_link_rate_8bits = btc_tx_link_rate;
	assign rx_link_rate_8bits = btc_rx_link_rate;
	
	wire [1:0] tx_link_rate_2b_enc = btc_tx_link_rate == 8'h14 ? 2'b10 : (btc_tx_link_rate == 8'h0a ? 2'b01 : 2'b00);
	wire [1:0] rx_link_rate_2b_enc = btc_rx_link_rate == 8'h14 ? 2'b10 : (btc_rx_link_rate == 8'h0a ? 2'b01 : 2'b00);

    //*************************************************************
    // Reconfig Mgmt interfaces
    //*************************************************************

    // Don't make any reconfiguration requests while any reconfig
    // operations are busy
    wire reconfig_busy;
    assign reconfig_busy = (|rx_cal_busy) | (|tx_cal_busy) | rx_reconfig_busy | tx_reconfig_busy | tx_analog_reconfig_busy;

    //*************************************************************
    // RX Reconfig Mgmt
    //*************************************************************

    wire rx_reconfig_ready;

    generate 
    begin
        // Check if this interface should be supported
        if (RX_SUPPORT_DP) 
        begin
            // Instantiate a reconfig controller
            // Trigger on reset or link rate change
            // Wait for reconfigs to end
            altera_dp_reconfig_ctrl # (
                .WIDTH(2)
            ) rx_reconfig (
                .clk(xcvr_mgmt_clk),
                .reset(reset),
                .trigger(!rx_reconfig_ready | (rx_link_rate_2b_enc != rx_link_rate)),
                .din(rx_link_rate_2b_enc),
                .dout(rx_link_rate),
                .ready(!reconfig_busy),
                .ack(rx_reconfig_ack),
                .busy(rx_reconfig_busy),
                .req(rx_reconfig_req),
                .done(rx_reconfig_ready)
            );
        end 
        else begin
            // Stub out the unconnected outputs
            assign rx_reconfig_req   = 1'b0;
            assign rx_reconfig_ready = 1'b1;
            assign rx_link_rate	     = 2'b0;
        end
    end
    endgenerate

    //*************************************************************
    // TX Reconfig Mgmt
    //*************************************************************

    wire tx_reconfig_ready;

    generate 
    begin
        // Check if this interface should be supported
        if (TX_SUPPORT_DP) 
        begin
            // Instantiate a reconfig controller
            // Trigger on reset or link rate change
            // Wait for reconfigs to end and RX to be reconfiged
            altera_dp_reconfig_ctrl # (
                .WIDTH(2)
            ) tx_reconfig (
                .clk(xcvr_mgmt_clk),
                .reset(reset),
                .trigger(!tx_reconfig_ready | (tx_link_rate_2b_enc != tx_link_rate)),
                .din(tx_link_rate_2b_enc),
                .dout(tx_link_rate),
                .ready(!reconfig_busy && rx_reconfig_ready),
                .ack(tx_reconfig_ack),
                .busy(tx_reconfig_busy),
                .req(tx_reconfig_req),
                .done(tx_reconfig_ready)
            );
        end
        else begin
            // Stub out the unconnected outputs
            assign tx_reconfig_req   = 1'b0;
            assign tx_reconfig_ready = 1'b1;
            assign tx_link_rate	     = 2'b0;
        end 
    end
    endgenerate

    //*************************************************************
    // TX Analog Reconfig Mgmt
    //*************************************************************

    wire tx_analog_reconfig_ready;

    generate 
    begin
        // Check if this interface should be supported
        if (TX_SUPPORT_DP && TX_SUPPORT_ANALOG_RECONFIG) 
        begin
            // Instantiate a reconfig controller
            // Trigger on reset or do_analog_reconfig
            // Wait for reconfigs to end and RX & TX to be reconfiged
            altera_dp_reconfig_ctrl # (
                .WIDTH(4*TX_MAX_LANE_COUNT)
            ) tx_reconfig (
                .clk(xcvr_mgmt_clk),
                .reset(reset),
                .trigger(!tx_analog_reconfig_ready | btc_tx_analog_reconfig_req),
                .din({btc_tx_emp, btc_tx_vod}),
                .dout({tx_emp, tx_vod}),
                .ready(!reconfig_busy && rx_reconfig_ready && tx_reconfig_ready),
                .ack(tx_analog_reconfig_ack),
                .busy(tx_analog_reconfig_busy),
                .req(tx_analog_reconfig_req),
                .done(tx_analog_reconfig_ready)
            );
        end
        else begin
            // Stub out the unconnected outputs
            assign tx_analog_reconfig_req   = 1'b0;
            assign tx_analog_reconfig_ready = 1'b1;
            assign tx_vod                   = {TX_MAX_LANE_COUNT*2{1'b0}};
            assign tx_emp                   = {TX_MAX_LANE_COUNT*2{1'b0}};
        end 
    end
    endgenerate

endmodule

`default_nettype wire

